package com.gic.haoban.manage.service.task;

import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONObject;
import com.gic.commons.util.GICMQClientUtil;
import com.gic.haoban.manage.api.dto.DealParamMqDTO;
import com.gic.haoban.manage.api.enums.PreDealStatusEnum;
import com.gic.haoban.manage.api.enums.PreDealTypeEnum;
import com.gic.haoban.manage.api.enums.SyncTaskStatusEnum;
import com.gic.haoban.manage.api.service.DealSyncOperationApiService;
import com.gic.haoban.manage.api.service.MemberUnionidRelatedApiService;
import com.gic.haoban.manage.service.entity.TabHaobanPreDealLog;
import com.gic.haoban.manage.service.entity.TabHaobanSyncTask;
import com.gic.haoban.manage.service.service.PreDealService;
import com.gic.haoban.manage.service.service.SyncTaskService;
import com.gic.mq.sdk.GicMQClient;
import com.gic.redis.data.util.RedisUtil;

@Service
public class FriendClerkSyncOperation implements BaseSyncOperation {

	private static final Logger logger = LoggerFactory.getLogger(FriendClerkSyncOperation.class);
	@Autowired
	private PreDealService preDealService;
	@Autowired
	private MemberUnionidRelatedApiService memberUnionidRelatedApiService;
	@Autowired
	private SyncTaskService syncTaskService;
	
	@Override
	public void dealSingleByMq(DealParamMqDTO dealParamMqDTO, TabHaobanPreDealLog dataPre) {
		String wxEnterpriseId = dataPre.getWxEnterpriseId();
		String wxUserId = dataPre.getDataId();
		memberUnionidRelatedApiService.cleanNoExsitFriend(wxEnterpriseId, wxUserId);
		List<String> selfUserIdList = memberUnionidRelatedApiService.listSelfExterialList(wxEnterpriseId, wxUserId);
		List<TabHaobanPreDealLog> dealLogList = selfUserIdList.stream().map(dto -> {
            TabHaobanPreDealLog dealLog = new TabHaobanPreDealLog();
            dealLog.setDataId(dataPre.getDataId());
            dealLog.setpDataId(dto);
            dealLog.setDataType(PreDealTypeEnum.friend.getVal());
            dealLog.setStatusFlag(0);
            dealLog.setTaskId(dataPre.getTaskId());
            dealLog.setDataContent(JSONObject.toJSONString(dto));
            dealLog.setWxEnterpriseId(wxEnterpriseId);
            return dealLog;
        }).collect(Collectors.toList());

        preDealService.insert(dealLogList);
        HashSet<String> hashSet = new HashSet<>();
        hashSet.add("1");
        dealDepartmentToMq(dataPre.getTaskId(), hashSet, SyncTaskStatusEnum.friend_sync);
	}

	@Override
	public void dealException(String taskId, String dataId, String enterpriseId, String reason) {
		preDealService.updateStatusByDataId(taskId, dataId, PreDealStatusEnum.exception.getVal(), reason);

	}

	@Override
	public void dealSuccess(String taskId, String dataId, String enterpriseId, String wxEnterpriseId) {
		preDealService.updateStatusByDataId(taskId, dataId, PreDealStatusEnum.computed.getVal(), "成功");

	}

	@Override
	public void checkDepartmentTask(String taskId) {
		boolean b = preDealService.checkTask(taskId, PreDealTypeEnum.friend.getVal());
        if (!b) {
            return;
        }
        logger.info("好友同步成功:{}", taskId);
        TabHaobanSyncTask syncTask = syncTaskService.getSyncTask(taskId);
        //同步成功 进入门店处理
        if (syncTask.getStatusFlag().equals(SyncTaskStatusEnum.friend_sync.getVal())) {
            int errCount = preDealService.countByTaskId(taskId, -1, PreDealStatusEnum.exception.getVal());
            if (errCount > 0) {
                syncTaskService.updateTaskStatus(taskId, SyncTaskStatusEnum.exception_compute.getVal());
            } else {
                syncTaskService.updateTaskStatus(taskId, SyncTaskStatusEnum.compute.getVal());
            }
        }

	}
	
	/**
     * 放入mq处理部门数据
     *
     * @param taskId
     * @param dealList
     */
    private void dealDepartmentToMq(String taskId, Set<String> dealList, SyncTaskStatusEnum syncTaskStatusEnum) {
        //预处理分组任务
        syncTaskService.updateTaskStatus(taskId, syncTaskStatusEnum.getVal());
        dealList.forEach(relationId -> {
            DealParamMqDTO dealParamMqDTO = new DealParamMqDTO();
            dealParamMqDTO.setTaskId(taskId);
            dealParamMqDTO.setData(relationId);
            dealParamMqDTO.setType(syncTaskStatusEnum.getVal());
            GicMQClient clientInstance = GICMQClientUtil.getClientInstance();
            try {
                clientInstance.sendMessage("departmentSyncDealMq", JSONObject.toJSONString(dealParamMqDTO));
            } catch (Exception e) {
                logger.info("发送失败:{},{}", taskId, relationId);
                e.printStackTrace();
            }
        });
    }

}
